import { Autocomplete, AutocompleteRenderInputParams, List } from '@mui/material';
import React, { HTMLAttributes, PropsWithChildren, useCallback, useEffect, useRef, useState } from 'react';

import { ClearButton } from './helper/ClearButton';
import { renderGroupBy, renderGroupName, renderOptionLabel } from './helper/renderGroupName';
import { SeeMoreButton } from './helper/SeeMoreButton';
import { StyledTextField } from './NsAutocomplete.styled';
import { ComponentBoxProps, NsAutocompleteProps, OptionsProps } from './types.d';

const NsAutocomplete = ({
    id = 'custom_autocomplete_dropdown',
    options,
    OptionsUI,
    loading,
    noOptionsText,
    onSelect,
    onChange,
    clear,
    seeMore,
    placeholder,
    className,
    iconStart,
    iconEnd,
    inputValue,
    value,
    isFilterEnabled = true,
}: PropsWithChildren<NsAutocompleteProps>) => {
    const [scrollTopPosition, setScrollTopPosition] = useState(0);
    const listboxRef = useRef<HTMLUListElement | null>(null);

    useEffect(() => {
        if (listboxRef.current && options.length > 0) {
            listboxRef.current.scrollTop = scrollTopPosition;
        }
    }, [options, scrollTopPosition]);

    const handleSelect = useCallback(
        (event: React.SyntheticEvent, value: string | OptionsProps | null) => {
            if (typeof onSelect === 'function') {
                onSelect(value);
            }
        },
        [onSelect]
    );

    const onInputChange = useCallback(
        (event: React.SyntheticEvent, newInputValue: string) => {
            if (typeof onChange === 'function') {
                onChange(newInputValue);
            }
        },
        [onChange]
    );

    const renderInput = (params: AutocompleteRenderInputParams) => (
        <StyledTextField
            {...params}
            placeholder={placeholder}
            InputProps={{
                ...params.InputProps,
                className: 'styled-text-field',
                startAdornment: iconStart,
                endAdornment: iconEnd,
            }}
        />
    );

    const renderOption = (props: ComponentBoxProps, option: OptionsProps) => (
        <li {...props} key={props.id}>
            {OptionsUI ? <OptionsUI {...props} option={option} /> : null}
        </li>
    );

    const renderListBoxComponent = ({ children, ...otherProps }: ComponentBoxProps) => (
        <List {...otherProps} ref={listboxRef}>
            <ClearButton show={clear?.show} text={clear?.text} onClear={clear?.onClear} />
            {children}
            <SeeMoreButton
                onSeeMore={seeMore?.onSeeMore}
                show={seeMore?.show}
                text={seeMore?.text}
                options={options}
                setScrollTopPosition={setScrollTopPosition}
                listboxRef={listboxRef}
            />
        </List>
    );

    return (
        <Autocomplete
            id={id}
            disablePortal
            className={className}
            options={options}
            loading={loading}
            noOptionsText={noOptionsText}
            groupBy={renderGroupBy}
            getOptionLabel={renderOptionLabel}
            renderOption={renderOption}
            renderInput={renderInput}
            ListboxComponent={renderListBoxComponent}
            ListboxProps={{ scrollTopPosition } as HTMLAttributes<HTMLUListElement>} // Pass the current scroll position
            renderGroup={renderGroupName}
            onChange={handleSelect}
            onInputChange={onInputChange}
            value={value}
            inputValue={inputValue}
            filterOptions={
                isFilterEnabled
                    ? undefined // Use default filtering behavior
                    : (options) => options
            }
            freeSolo
        />
    );
};

export default NsAutocomplete;
